<?php

namespace Drupal\strava_clubs\Form;

use Drupal\Core\Entity\ContentEntityConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\strava\Api\Strava;
use Strava\API\Exception;

/**
 * Provides a form for refreshing Club entities.
 *
 * @ingroup strava
 */
class ClubRefreshForm extends ContentEntityConfirmFormBase {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $form['actions']['submit']['#value'] = $this->t('Refresh club details from Strava.');

    return $form;
  }

  /**
   * @inheritdoc
   */
  public function getDescription() {
    return $this->t('This action retrieves all club details from Strava and overwrites your local changes.');
  }

  /**
   * @inheritDoc
   */
  public function getQuestion() {
    return $this->t('Do you want get the refresh club details from Strava?');
  }

  /**
   * @inheritDoc
   */
  public function getCancelUrl() {
    return Url::fromRoute('entity.club.canonical', [
      'club' => $this->getEntity()
        ->id(),
    ]);
  }

  /**
   * Refresh entity info from Strava API.
   *
   * @throws \Strava\API\Exception
   */
  public function refreshEntity() {
    /** @var \Drupal\strava_clubs\Entity\Club $entity */
    $entity = $this->getEntity();

    // @TODO: figure out a way to select an Athlete for this client call.
    $strava = new Strava();
    /** @var \Strava\API\Client $client */
    $client = $strava->getApiClientForAthlete($entity);
    if ($client) {
      $club_details = $client->getClub($entity->id());
      \Drupal::service('strava.club_manager')
        ->updateClub($club_details);
    }
  }

  /**
   * @inheritdoc
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      $this->refreshEntity();
      $form_state->setRedirect(
        'entity.club.canonical',
        ['club' => $this->entity->id()]
      );
    }
    catch (Exception $e) {
      $form_state->setRedirect(
        'entity.club.refresh',
        ['club' => $this->entity->id()]
      );
      $this->messenger()->addError($e->getMessage());
    }
  }
}
